# -*- coding: utf-8 -*-

from django.db import models
from phonenumber_field.modelfields import PhoneNumberField


class Signature(models.Model):
    name = models.CharField('Имя', max_length=128)

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ('name',)
        verbose_name = 'Подпись'
        verbose_name_plural = 'Подписи сообщений'


class SMSMessage(models.Model):
    STATUS_CONNECTION = 'connection error'
    STATUS_ACCEPTED = 'accepted'
    STATUS_EMPTY_FIELD = 'empty field. reject.'
    STATUS_INCORRECT_USER = 'incorrect user or password. reject'
    STATUS_NO_CREDITS = 'no credits'
    STATUS_INCORRECT_NAME = 'incorrect sender name. reject'
    STATUS_INCORRECT_ADRESS = 'incorrect destination adress. reject'
    STATUS_INCORRECT_DATE = 'incorrect date. reject'
    STATUS_DELIVERY_SUCCESS = 'delivery success'
    STATUS_DELIVERY_FAILURE = 'delivery failure'
    STATUS_SMSC_SUBMIT = 'smsc submit'
    STATUS_SMSC_REJECT = 'smsc reject'
    STATUS_QUEUE = 'queue'
    STATUS_WAIT_STATUS = 'wait status'
    STATUS_INCORRECT_ID = 'incorrect id. reject'

    STATUS_CHOICES = (
        (STATUS_CONNECTION, 'Ошибка при соединение с сервером'),
        (STATUS_ACCEPTED, 'Сообщение принято сервисом'),
        (STATUS_EMPTY_FIELD, 'Не все обязательные поля заполнены'),
        (STATUS_INCORRECT_USER, 'Ошибка авторизации'),
        (STATUS_NO_CREDITS, ' 	Недостаточно SMS на балансе'),
        (STATUS_INCORRECT_NAME, 'Неверная (незарегистрированная) подпись отправителя'),
        (STATUS_INCORRECT_ADRESS, 'Неверно задан номер тефона (формат 71234567890)'),
        (STATUS_INCORRECT_DATE, 'Неправильный формат дат'),
        (STATUS_DELIVERY_SUCCESS, 'Сообщение доставлено'),
        (STATUS_DELIVERY_FAILURE, 'Ошибка доставки SMS'),
        (STATUS_SMSC_SUBMIT, 'Сообщение доставлено в SMSC'),
        (STATUS_SMSC_REJECT, 'Отвергнуто SMSC'),
        (STATUS_QUEUE, ' 	Ожидает отправки'),
        (STATUS_WAIT_STATUS, 'Ожидание статуса (запросите позднее)'),
        (STATUS_INCORRECT_ID, 'Неверный идентификатор сообщения'),
    )

    phone = PhoneNumberField('Телефон')
    signature = models.ForeignKey(Signature, verbose_name='Подпись')
    text = models.TextField('Текст')
    sms_id = models.IntegerField('ID SMS', db_index=True, blank=True, null=True, editable=False)
    status = models.CharField('Статус', max_length=128, choices=STATUS_CHOICES)
    created = models.DateTimeField('Создано', auto_now_add=True)

    def __unicode__(self):
        return u'{0} <{1}>'.format(self.phone, self.get_status_display())

    class Meta:
        ordering = ('-created', )
        verbose_name = 'Сообщение'
        verbose_name_plural = 'SMS сообщения'


# -*- coding: utf-8 -*-

from django.conf import settings

SMSAERO_USER = getattr(settings, 'SMSAERO_USER', '')
SMSAERO_PASSWORD = getattr(settings, 'SMSAERO_PASSWORD', '')
SMSAERO_PASSWORD_MD5 = getattr(settings, 'SMSAERO_PASSWORD_MD5', '')



# -*- coding: utf-8 -*-

import factory
from .models import Signature
from .models import SMSMessage


class SignatureF(factory.Factory):
    FACTORY_FOR = Signature

    name = factory.Sequence(lambda n: 'Name{0}'.format(n))


class SMSMessageF(factory.Factory):
    FACTORY_FOR = SMSMessage

    sms_id = factory.Sequence(lambda n: n)
    phone = factory.Sequence(lambda n: '+7123456789{0}'.format(n))
    signature = factory.LazyAttribute(lambda a: SignatureF())
    text = factory.Sequence(lambda n: 'Message{0}'.format(n))
    status = SMSMessage.STATUS_ACCEPTED


# -*- coding: utf-8 -*-

import re
import logging
import urllib2
from hashlib import md5
from django_rq import job
from urllib import quote_plus
from django.utils.html import strip_tags
from smsaero import conf as settings
from models import Signature
from models import SMSMessage

logger = logging.getLogger('smsaero')


class SmsSender():
    PROTOCOL = 'http'
    HOST = 'gate.smsaero.ru'

    def send_request(self, params, link):
        params['user'] = settings.SMSAERO_USER
        params['password'] = self._get_password()

        url = '%s://%s%s?%s' % (
            self.PROTOCOL,
            self.HOST,
            link,
            '&'.join('%s=%s' % i for i in params.items()),
        )
        try:
            response = urllib2.urlopen(url)
            return strip_tags(response.read()).strip()
        except urllib2.URLError:
            logger.error('connection error', exc_info=True)
            return 'connection error'
        except urllib2.HTTPErrorProcessor:
            logger.error('connection error', exc_info=True)
            return 'connection error'

    def _get_password(self):
        if settings.SMSAERO_PASSWORD_MD5:
            return settings.SMSAERO_PASSWORD_MD5

        m = md5()
        m.update(settings.SMSAERO_PASSWORD)
        return m.hexdigest()

    def parse_response(self, response):
        if '=' in response:
            return response.split('=')
        return response, None

    def get_signature(self, signature_id=None):
        if signature_id:
            try:
                return Signature.objects.get(pk=signature_id)
            except Signature.DoesNotExist:
                pass
        else:
            try:
                return Signature.objects.all()[0]
            except IndexError:
                pass
        return


sender = SmsSender()


def send_sms(phone, text, signature_id=None, date=None, link='/send/'):
    phone = phone.replace(' ', '')\
        .replace('-', '')\
        .replace('(', '')\
        .replace(')', '')

    match = re.search('^\+?\d{11}$', phone)
    if not match:
        Exception('Not valid phone')
        return

    signature = sender.get_signature(signature_id)
    if not signature:
        Exception('Have not not one signature')
        return

    params = {
        'to': phone.replace('+', ''),
        'text': quote_plus(text),
        'from': signature.name,
        'date': date or '',
    }
    response = sender.send_request(params, link)
    sms_id, status = sender.parse_response(response)

    if not sms_id or not status:
        msg = 'Can not send SMS to %s. Status: %s' % (phone, sms_id)
        logger.error(msg)
        raise Exception(msg)

    sms = SMSMessage(
        phone=phone,
        signature=signature,
        text=text,
        sms_id=sms_id,
        status=status,
    )
    sms.save()
    return sms


def get_sms_status(sms_id, link='/status/'):
    response = sender.send_request({'id': sms_id}, link)
    sms_id, status = sender.parse_response(response)
    return status


def get_balance(link='/balance/'):
    response = sender.send_request({}, link)
    stuff, balanse = sender.parse_response(response)
    return balanse


def get_signatures_name(link='/senders/'):
    response = sender.send_request({}, link)
    return [n for n in response.split('\n') if n]


@job('default')
def send_sms_async(*args, **kwargs):
    return send_sms(*args, **kwargs)


@job('default')
def get_sms_status_async(*args, **kwargs):
    return get_sms_status(*args, **kwargs)


@job('default')
def get_balance_async(*args, **kwargs):
    return get_balance(*args, **kwargs)


@job('default')
def get_signatures_name_async(*args, **kwargs):
    return get_signatures_name(*args, **kwargs)



# -*- coding: utf-8 -*-

import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

tests_require = [
    'Django>=1.3',
    'nose>1.3',
    'mock==1.0.1',
    'coveralls',
    'factory_boy',
    'django-phonenumber-field',
]

setup(
    name='django-smsaero',
    version='0.9.2',
    packages=['smsaero'],
    include_package_data=True,
    license='MIT',
    description='A simple Django app for send SMS via smsaero.ru.',
    long_description=README,
    url='http://github.com/drmartiner/django-smsaero/',
    author='DrMartiner',
    author_email='DrMartiner@GMail.Com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'urllib3',
        'factory_boy',
        'django-phonenumber-field',
    ],
)


# -*- coding: utf-8 -*-

from django.contrib import admin
from .models import Signature
from .models import SMSMessage


class SMSMessageAdmin(admin.ModelAdmin):
    list_display = ('phone', 'status', 'signature', 'sms_id', 'created')
    list_filter = ('signature', 'status', 'created')
    list_display_links = ('signature',)
    search_fields = ('text', 'signature__name')

    can_delete = False

    def has_add_permission(self, request):
        return False

    # TODO: send after saving if selected check


admin.site.register(Signature)
admin.site.register(SMSMessage, SMSMessageAdmin)


# -*- coding: utf-8 -*-

from mock import patch
from hashlib import md5
from django.test import TestCase
from .utils import SmsSender
from .utils import send_sms
from .utils import get_sms_status
from .utils import get_balance
from .utils import get_signatures_name
from .models import SMSMessage
from .factories import SignatureF
from .factories import SMSMessageF


class SmsSenderTest(TestCase):
    def _fake_urlopen(self):
        class FakeStreamReader(object):
            def read(self):
                return '123=accepted'
        return FakeStreamReader()

    @patch('smsaero.conf.SMSAERO_PASSWORD', 'FAKE')
    @patch('urllib2.urlopen', _fake_urlopen)
    def test_send_request(self):
        sender = SmsSender()
        response = sender.send_request({}, '/link/')
        self.assertIn(SMSMessage.STATUS_ACCEPTED, response)

    @patch('smsaero.conf.SMSAERO_PASSWORD', 'FAKE')
    @patch('smsaero.conf.SMSAERO_PASSWORD_MD5', '')
    def test_get_encrypted_password(self):
        sender = SmsSender()
        hashed = sender._get_password()

        m = md5()
        m.update('FAKE')
        passord = m.hexdigest()

        self.assertEqual(hashed, passord, 'Encrypted pass not equal with MD5 hash')

    @patch('smsaero.conf.SMSAERO_PASSWORD_MD5', 'md5_hash')
    def test_get_no_encrypted_password(self):
        sender = SmsSender()
        hashed = sender._get_password()
        self.assertEqual(hashed, 'md5_hash', 'MD5 hash of password not equal')

    def test_parse_response_key_val(self):
        sender = SmsSender()
        key, val = sender.parse_response('key=val')

        self.assertEqual(key, 'key', 'Did not parse key from response')
        self.assertEqual(val, 'val', 'Did not parse value from response')

    def test_parse_response_key_only(self):
        sender = SmsSender()
        key, stuff = sender.parse_response('key')

        self.assertEqual(key, 'key', 'Did not parse key from response')
        self.assertIsNone(stuff, 'Value is not empty at parse of response without "="')

    def test_get_signature_by_id(self):
        sign = SignatureF()
        sign.save()
        sender = SmsSender()

        signature = sender.get_signature(sign.id)

        self.assertEqual(signature.id, sign.id)

    def test_get_first_signature(self):
        sign = SignatureF()
        sign.save()
        sender = SmsSender()

        signature = sender.get_signature()

        self.assertEqual(signature.id, sign.id)


class SmsAeroAPITest(TestCase):
    def _get_sms(self, to, text):
        return '0=accepted'

    def _get_status(self, link, params):
        return '0=delivery success'

    def _get_balance(self, link, params):
        return 'balance=12345'

    def _get_signatures_name(self, link, params):
        return 'Sender_one\nSender_two\nSender_three'

    @patch('smsaero.utils.SmsSender.send_request', _get_sms)
    def test_send_sms(self):
        signature = SignatureF()
        signature.save()

        self._send_sms('+71234567890', 'Message0', signature)
        self._send_sms('+7 (123) 456-78-90', 'Message1', signature)

        signature.delete()

    def _send_sms(self, phone, message, signature):
        sent_sms = send_sms(phone, message)
        sms = SMSMessageF()

        self.assertEquals(sent_sms.text, sms.text, 'Not equals text of sent SMS')
        self.assertEquals(sent_sms.status, SMSMessage.STATUS_ACCEPTED, 'Status of sent SMS is not ACCEPTED')
        self.assertEquals(sent_sms.signature, signature, 'Not equals signature of sent SMS')

    @patch('smsaero.utils.SmsSender.send_request', _get_sms)
    def test_send_to_invalid_number(self):
        signature = SignatureF()
        signature.save()

        sent_sms = send_sms('+123', 'Message0')
        self.assertIsNone(sent_sms, 'Sms was sent to invalid number')

    @patch('smsaero.utils.SmsSender.send_request', _get_status)
    def test_get_sms_status(self):
        sms_status = get_sms_status(0)

        self.assertEquals(sms_status, SMSMessage.STATUS_DELIVERY_SUCCESS, 'SMS status is not DELIVERY_SUCCESS')


    @patch('smsaero.utils.SmsSender.send_request', _get_balance)
    def test_get_balance(self):
        sms_balance = get_balance()
        balance = '12345'

        self.assertEquals(sms_balance, balance, 'Not equals SMS balance')


    @patch('smsaero.utils.SmsSender.send_request', _get_signatures_name)
    def test_get_signatures_name(self):
        signatures_name = get_signatures_name()
        signatures_fake = ['Sender_one', 'Sender_two', 'Sender_three']

        self.assertEquals(signatures_name, signatures_fake, 'Not equals signatures')
